﻿using System;
using System.Configuration;
using KRE.Replicate.Contracts.Data;
using KRE.Replicate.Contracts.Services;
using log4net;

namespace KRE.Replicate.Client
{
    public class ConflictMaker
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(ConflictMaker));

        private const string Primary = "Primary";
        private const string Secondary = "Secondary";

        private readonly DataFactory _dataFactory;

        public ConflictMaker()
        {
            _dataFactory = new DataFactory();
        }

        public void MakeConflict()
        {
            Log.Debug("Open firewall, setup replication, put original data");
            SetRavenFirewall(true);

            SetupReplication(Primary, new RavenNode(ConfigurationManager.AppSettings["secondary"], "Ravn"));
            SetupReplication(Secondary, new RavenNode(ConfigurationManager.AppSettings["primary"], "Ravn"));

            PutBar(Primary, _dataFactory.Bar1);
            PutFoo(Primary, _dataFactory.Foo1);

            Log.Debug("Close firewall, push conflicting data");
            SetRavenFirewall(false);

            PutBar(Primary, _dataFactory.Bar2);
            PutFoo(Primary, _dataFactory.Foo2);

            PutBar(Secondary, _dataFactory.Bar3);
            PutFoo(Secondary, _dataFactory.Foo3);

            Log.Debug("Open firewall to allow replication");
            SetRavenFirewall(true);
        }

        public static void SetRavenFirewall(bool enable)
        {
            RavenFirewallRule(Primary, enable);
            RavenFirewallRule(Secondary, enable);
        }

        private static void RavenFirewallRule(string serverName, bool enable)
        {
            using (var serviceChannel = ServiceChannel<IManagementService>.Get(serverName))
            {
                serviceChannel.Service.SetFirewallRuleForRaven(enable);
            }
        }

        private static void SetupReplication(string from, RavenNode to)
        {
            using (var serviceChannel = ServiceChannel<IManagementService>.Get(from))
            {
                serviceChannel.Service.ReplicteTo(to);
            }
        }

        private static void PutBar(string serverName, Bar bar)
        {
            using (var serviceChannel = ServiceChannel<IBarService>.Get(serverName))
            {
                serviceChannel.Service.AddBar(bar);
            }
        }

        private static void PutFoo(string serverName, Foo foo)
        {
            using (var serviceChannel = ServiceChannel<IFooService>.Get(serverName))
            {
                serviceChannel.Service.AddFoo(foo);
            }
        }

        private static void DeleteBar(string serverName, int id)
        {
            using (var serviceChannel = ServiceChannel<IBarService>.Get(serverName))
            {
                serviceChannel.Service.DeleteBar(id);
            }
        }

        private static void DeleteFoo(string serverName, int id)
        {
            using (var serviceChannel = ServiceChannel<IFooService>.Get(serverName))
            {
                serviceChannel.Service.DeleteFoo(id);
            }
        }
    }
}
