﻿using System;
using KRE.Replicate.Contracts.Services;
using log4net;
using log4net.Config;

namespace KRE.Replicate.Client
{
    public class Program
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(Program));

        private const string Primary = "Primary";
        private const string Secondary = "Secondary";

        public static void Main(string[] args)
        {
            XmlConfigurator.Configure();

            try
            {
                ResetServer(Primary);
                ResetServer(Secondary);

                new ConflictMaker().MakeConflict();

                // Add a short delay to let Raven do the replication
                Console.WriteLine("Press enter to do a read call. This will:");
                Console.WriteLine("- Block firewalls again (so that both dbs will have to do their own conflict resolution)");
                Console.Write("- Do a read call to trigger conflict resolution");
                Console.ReadLine();

                ConflictMaker.SetRavenFirewall(false);

                PrintDataFrom(Primary);
                PrintDataFrom(Secondary);
                Log.Info("Done");

                Console.WriteLine("Press enter to reopen the firewalls");
                Console.Write("PS: This might cause new conflicts in raven if the two nodes resolved conflicts differently");
                Console.ReadLine();

                ConflictMaker.SetRavenFirewall(true);
            }
            catch (Exception e)
            {
                Log.Error("Replication client threw exception: ", e);
            }

            Console.WriteLine("Done. Press enter to exit");
            Console.ReadLine();
        }

        private static void ResetServer(string serverName)
        {
            using (var serviceChannel = ServiceChannel<IManagementService>.Get(serverName))
            {
                var service = serviceChannel.Service;

                service.Reset();
            }
        }

        private static void PrintDataFrom(string serverName)
        {
            Log.Info(serverName);

            using (var serviceChannel = ServiceChannel<IBarService>.Get(serverName))
            {
                var service = serviceChannel.Service;

                Console.WriteLine(string.Join(", ", service.GetAllBars()));
            }

            using (var serviceChannel = ServiceChannel<IFooService>.Get(serverName))
            {
                var service = serviceChannel.Service;

                Console.WriteLine(string.Join(", ", service.GetAllFoos()));
            }
        }
    }
}
