﻿using System;
using System.ServiceModel;
using log4net;

namespace KRE.Replicate.Client
{
    public class ServiceChannel<T> : IDisposable
    {
        private static readonly ILog Log = LogManager.GetLogger($"{typeof(ServiceChannel<T>).Namespace}.{typeof(ServiceChannel<T>).Name}");

        public T Service { get; private set; }
        private readonly ChannelFactory<T> _channelFactory;

        public ServiceChannel(T service, ChannelFactory<T> channelFactory)
        {
            Service = service;
            _channelFactory = channelFactory;
        }

        public void Dispose()
        {
            _channelFactory.Close();
            ((IDisposable) _channelFactory)?.Dispose();
        }

        internal static ServiceChannel<T> Get(string endpointName)
        {
            var channelFactory = new ChannelFactory<T>($"{typeof(T).Name}_{endpointName}");
            channelFactory.Open();
            var service = channelFactory.CreateChannel();

            return new ServiceChannel<T>(service, channelFactory);
        }
    }
}
