﻿using System;
using System.Linq;
using KRE.Replicate.Contracts.Data;
using log4net;
using Raven.Abstractions.Data;
using Raven.Client.Listeners;
using Raven.Imports.Newtonsoft.Json;
using Raven.Json.Linq;

namespace KRE.Replicate.Server.Database.ConflictResolvers
{
    public class BarResolver : IDocumentConflictListener
    {
        protected static readonly ILog Log = LogManager.GetLogger(typeof(BarResolver));

        public bool TryResolveConflict(string key, JsonDocument[] conflictedDocs, out JsonDocument resolvedDocument)
        {
            if (!key.StartsWith("bars/"))
            {
                resolvedDocument = null;
                return false;
            }

            resolvedDocument = conflictedDocs
                .OrderBy(d => d.LastModified)
                .First();

            var bar = conflictedDocs
                .Select(Deserialize)
                .Aggregate(Merge);

            resolvedDocument.DataAsJson = RavenJObject.FromObject(bar);
            resolvedDocument.LastModified = DateTime.Now;

            Log.Debug($"Resolved conflict in {key}");
            resolvedDocument.Metadata.Remove("Raven-Replication-Conflict-Document");
            resolvedDocument.Metadata.Remove("Raven-Replication-Conflict");
            resolvedDocument.Metadata.Remove("@id");
            resolvedDocument.Metadata.Remove("@etag");
            return true;
        }

        private static Bar Deserialize(JsonDocument json)
        {
            return JsonConvert.DeserializeObject<Bar>(json.DataAsJson.ToString());
        }

        public static Bar Merge(Bar b1, Bar b2)
        {
            b1.LastModified = DateTime.Now;
            b1.Data = $"{b1.Data} {b2.Data}";
            return b1;
        }
    }
}
