﻿using System;
using System.Collections.Generic;
using System.Linq;
using KRE.Replicate.Contracts.Data;
using log4net;
using Raven.Abstractions.Data;
using Raven.Client.Listeners;
using Raven.Imports.Newtonsoft.Json;
using Raven.Json.Linq;

namespace KRE.Replicate.Server.Database.ConflictResolvers
{
    public class FooResolver : IDocumentConflictListener
    {
        protected static readonly ILog Log = LogManager.GetLogger(typeof(FooResolver));

        public bool TryResolveConflict(string key, JsonDocument[] conflictedDocs, out JsonDocument resolvedDocument)
        {
            if (!key.StartsWith("foos/"))
            {
                resolvedDocument = null;
                return false;
            }

            resolvedDocument = conflictedDocs
                .OrderBy(d => d.LastModified)
                .First();

            var foo = conflictedDocs
                .Select(Deserialize)
                .Aggregate(Merge);

            resolvedDocument.DataAsJson = RavenJObject.FromObject(foo);
            resolvedDocument.LastModified = DateTime.Now;

            Log.Debug($"Resolved conflict in {key}");
            resolvedDocument.Metadata.Remove("Raven-Replication-Conflict-Document");
            resolvedDocument.Metadata.Remove("Raven-Replication-Conflict");
            resolvedDocument.Metadata.Remove("@id");
            resolvedDocument.Metadata.Remove("@etag");
            return true;
        }

        private static Foo Deserialize(JsonDocument json)
        {
            return JsonConvert.DeserializeObject<Foo>(json.DataAsJson.ToString());
        }

        public static Foo Merge(Foo f1, Foo f2)
        {
            return new Foo
            {
                Id = f1.Id,
                LastModified = DateTime.Now,
                ListInt = f1.ListInt.Union(f2.ListInt).ToList(),
                DictionaryIntString = DictionaryMerge(f1.DictionaryIntString, f2.DictionaryIntString),
                Bar = BarResolver.Merge(f1.Bar, f2.Bar),
                ListBar = MergeListBars(f1.ListBar, f2.ListBar)
            };
        }

        private static Dictionary<int, string> DictionaryMerge(Dictionary<int, string> fd1, Dictionary<int, string> fd2)
        {
            return fd1.Concat(fd2)
                .GroupBy(p => p.Key)
                .Select(g => g.Aggregate((a, b) => new KeyValuePair<int, string>(a.Key, $"{a.Value},{b.Value}")))
                .ToDictionary(x => x.Key, y => y.Value);
        }

        private static List<Bar> MergeListBars(IEnumerable<Bar> bs1, IEnumerable<Bar> bs2)
        {
            return bs1.Concat(bs2)
                .GroupBy(b => b.Id)
                .Select(g => g.Aggregate(BarResolver.Merge))
                .ToList();
        }
    }
}
