﻿using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using KRE.Replicate.Common;
using KRE.Replicate.Contracts.Data;
using KRE.Replicate.Contracts.Services;
using log4net;
using Raven.Client;

namespace KRE.Replicate.Server.Services
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall,
        ConcurrencyMode = ConcurrencyMode.Multiple,
        UseSynchronizationContext = false,
        Name = "BarService",
        Namespace = "http://replicate.kryel.com/v1/Bar")]
    public class BarService : IBarService
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(BarService));

        public void AddBar(Bar bar)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                session.Store(bar, $"bars/{bar.Id}");
                session.SaveChanges();
            }
        }

        public void AddBars(List<Bar> bars)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                foreach (var bar in bars)
                {
                    session.Store(bar, $"bars/{bar.Id}");
                }
                session.SaveChanges();
            }
        }

        public Bar GetBar(int id)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                return session.Load<Bar>(id);
            }
        }

        public List<Bar> GetAllBars()
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                return session.Query<Bar>()
                    .Customize(x => x.WaitForNonStaleResultsAsOfLastWrite())
                    .ToList();
            }
        }

        public void DeleteBar(int id)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                session.Delete<Bar>(id);

                session.SaveChanges();
            }
        }
    }
}
