﻿using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using KRE.Replicate.Common;
using KRE.Replicate.Contracts.Data;
using KRE.Replicate.Contracts.Services;
using log4net;
using Raven.Client;

namespace KRE.Replicate.Server.Services
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall,
        ConcurrencyMode = ConcurrencyMode.Multiple,
        UseSynchronizationContext = false,
        Name = "FooService",
        Namespace = "http://replicate.kryel.com/v1/Foo")]
    public class FooService : IFooService
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(FooService));

        public void AddFoo(Foo foo)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                session.Store(foo, $"foos/{foo.Id}");
                session.SaveChanges();
            }
        }

        public void AddFoos(List<Foo> foos)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                foreach (var foo in foos)
                {
                    session.Store(foo, $"foos/{foo.Id}");
                }
                session.SaveChanges();
            }
        }

        public Foo GetFoo(int id)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                return session.Load<Foo>(id);
            }
        }

        public List<Foo> GetAllFoos()
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                return session.Query<Foo>()
                    .Customize(x => x.WaitForNonStaleResultsAsOfLastWrite())
                    .ToList();
            }
        }

        public void DeleteFoo(int id)
        {
            using (var session = NinjectCommon.GetService<IDocumentSession>())
            {
                session.Delete<Foo>(id);

                session.SaveChanges();
            }
        }
    }
}
