﻿using System;
using System.ServiceModel;
using KRE.Replicate.Common;
using KRE.Replicate.Contracts.Data;
using KRE.Replicate.Contracts.Services;
using KRE.Replicate.Server.Database;
using log4net;
using NetFwTypeLib;
using Raven.Client;

namespace KRE.Replicate.Server.Services
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall,
        ConcurrencyMode = ConcurrencyMode.Multiple,
        UseSynchronizationContext = false,
        Name = "ManagementService",
        Namespace = "http://replicate.kryel.com/v1/Management")]
    public class ManagementService : IManagementService
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(ManagementService));

        public void Reset()
        {
            var raven = NinjectCommon.GetService<IRaven>();
            raven.ResetRaven();

            try
            {
                using (var session = NinjectCommon.GetService<IDocumentSession>())
                {
                    session.Store(new {Initialized = true}, "Raven/MyData/InitStatus");
                    session.SaveChanges();
                }
            }
            catch (Exception e)
            {
                Log.Debug("Store Initialized=true failed: ", e);
            }
        }

        public void ReplicteTo(RavenNode other)
        {
            NinjectCommon.GetService<IRaven>().StartReplication(other);
        }

        public void SetFirewallRuleForRaven(bool enable)
        {
            var typeFromProgId = Type.GetTypeFromProgID("HNetCfg.FwPolicy2");
            var policy = (INetFwPolicy2)Activator.CreateInstance(typeFromProgId);

            var ravenRule = policy.Rules.Item("Raven 8080");
            ravenRule.Enabled = enable;
        }
    }
}
