﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceProcess;
using KRE.Replicate.Common;
using KRE.Replicate.Server.Database;
using KRE.Replicate.Server.Services;
using log4net;
using Ninject;

namespace KRE.Replicate.Service
{
    public class WindowsService : ServiceBase
    {
        private static readonly ILog Log = LogManager.GetLogger(typeof(WindowsService));

        private readonly List<Tuple<Type, ServiceHost>> _hostedServices = new List<Tuple<Type, ServiceHost>>();

        protected override void OnStart(string[] args)
        {
            base.OnStart(args);
            StartServices();
        }

        protected override void OnStop()
        {
            base.OnStop();
            StopServices();
        }

        public void StartFake()
        {
            StartServices();
        }

        public void StopFake()
        {
            StopServices();
        }

        private void StartServices()
        {
            KernelSetup();

            ServiceName = "KRE.Replicate Service Host";

            HostService(typeof(FooService));
            HostService(typeof(BarService));
            HostService(typeof(ManagementService));
        }

        private static void KernelSetup()
        {
            var kernel = new StandardKernel();
            NinjectCommon.Kernel = kernel;
            kernel.Bind<IRaven>().ToConstant(new TheRaven(kernel));
        }

        private void HostService(Type type)
        {
            var serviceHost = new ServiceHost(type);
            _hostedServices.Add(new Tuple<Type, ServiceHost>(type, serviceHost));

            serviceHost.Open();
            Log.Debug($"Opened {type.Name} at {string.Join(", ", serviceHost.BaseAddresses)}");
        }

        private void StopServices()
        {
            foreach (var hostedService in _hostedServices)
            {
                Log.Debug($"Closing {hostedService.Item1.Name}... Timeout is: {hostedService.Item2.CloseTimeout}");
                hostedService.Item2.Close();
                Log.Debug($"Closed {hostedService.Item1.Name}");
            }
        }
    }
}
